﻿using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace Falcon.Extend
{
    /// <summary>
    /// AES加密服务
    /// </summary>
    public class AESCryptoService:ICryptoService
    {
        /// <summary>
        /// 用AES加密字符串
        /// </summary>
        /// <param name="content">明文</param>
        /// <param name="key">关键字</param>
        /// <returns>密文</returns>
        public string Encrypt(string content,string key) {
            var encryptKey = GetDecryptKey(key);
            using(var aesAlg = Aes.Create()) {
                var iv = aesAlg.IV;
                using(var encryptor = aesAlg.CreateEncryptor(encryptKey,iv)) {
                    using(var msEncrypt = new MemoryStream()) {
                        using(var csEncrypt = new CryptoStream(msEncrypt,encryptor,CryptoStreamMode.Write))
                        using(var swEncrypt = new StreamWriter(csEncrypt)) {
                            swEncrypt.Write(content);
                        }
                        var decryptedContent = msEncrypt.ToArray();
                        var result = new byte[iv.Length + decryptedContent.Length];
                        Buffer.BlockCopy(iv,0,result,0,iv.Length);
                        Buffer.BlockCopy(decryptedContent,0,result,iv.Length,decryptedContent.Length);
                        return Convert.ToBase64String(result);
                    }
                }
            }
        }
        /// <summary>
        /// AES解密
        /// </summary>
        /// <param name="content">密文</param>
        /// <param name="key">关键字</param>
        /// <returns>明文</returns>
        public string Decrypt(string content,string key) {
            var fullCipher = Convert.FromBase64String(content);
            var iv = new byte[16];
            var cipher = new byte[fullCipher.Length - iv.Length];
            Buffer.BlockCopy(fullCipher,0,iv,0,iv.Length);
            Buffer.BlockCopy(fullCipher,iv.Length,cipher,0,fullCipher.Length - iv.Length);
            var decryptKey = GetDecryptKey(key);
            using(var aesAlg = Aes.Create()) {
                using(var decryptor = aesAlg.CreateDecryptor(decryptKey,iv)) {
                    string result;
                    using(var msDecrypt = new MemoryStream(cipher)) {
                        using(var csDecrypt = new CryptoStream(msDecrypt,decryptor,CryptoStreamMode.Read)) {
                            using(var srDecrypt = new StreamReader(csDecrypt)) {
                                result = srDecrypt.ReadToEnd();
                            }
                        }
                    }
                    return result;
                }
            }
        }

        /// <summary>
        /// 格式化加密key长度
        /// </summary>
        /// <param name="key">加密用的key</param>
        /// <param name="keyLength">格式化长度</param>
        /// <returns>加密key字节数组</returns>
        protected byte[] GetDecryptKey(string key,int keyLength = 32) {
            var buf = new byte[keyLength];
            var ek = Encoding.UTF8.GetBytes(key);
            var len = keyLength > ek.Length ? ek.Length : buf.Length;
            for(var i = 0;i < len;i++) {
                buf[i] = ek[i];
            }
            return buf;
        }
    }
}
